(* HCoop Domtool (http://hcoop.sourceforge.net/)
 * Copyright (c) 2006, Adam Chlipala
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*)

(* Main interface *)

structure Main :> MAIN = struct

open Ast Print

val dmy = ErrorMsg.dummyLoc

val tInit = (TAction ((CRoot, dmy),
		      StringMap.empty,
		      StringMap.empty),
	     dmy)
	    
fun check fname =
    let
	val prog = Parse.parse fname
    in
	if !ErrorMsg.anyErrors then
	    ()
	else
	    let
		val G' = Tycheck.checkFile Env.empty tInit prog
	    in
		()
	    end
    end

fun reduce fname =
    let
	val prog = Parse.parse fname
    in
	if !ErrorMsg.anyErrors then
	    ()
	else
	    let
		val G' = Tycheck.checkFile Env.empty tInit prog
	    in
		if !ErrorMsg.anyErrors then
		    ()
		else
		    case prog of
			(_, SOME body) =>
			let
			    val body' = Reduce.reduceExp G' body
			in
			    printd (PD.hovBox (PD.PPS.Rel 0,
					       [PD.string "Result:",
						PD.space 1,
						p_exp body']))
			end
		      | _ => ()
	    end
    end

fun eval fname =
    let
	val prog = Parse.parse fname
    in
	if !ErrorMsg.anyErrors then
	    ()
	else
	    let
		val G' = Tycheck.checkFile Env.empty tInit prog
	    in
		if !ErrorMsg.anyErrors then
		    ()
		else
		    case prog of
			(_, SOME body) => Eval.exec StringMap.empty body
		      | _ => ()
	    end
    end

end
